#
#   Multiple Response Variable [2023/10/19]
#
# Based on  userfriendlyscience::multiResponse
# 
# multipleResponse (data, 
#                   items = NULL, 
#                   regex = NULL, 
#                   endorsedOption = 1, 
#                   order=NULL, 
#                   plot=NULL, 
#                   names.arg=NULL, 
#                   ylab=NULL, ...) 
# 
# data:           Dataframe containing the variables to display
# items:          list of variable (column) name to process
# regex:          see userfriendlyscience::multiResponse help
# endorsedOption: value that represents the endorsed option 
# order:          should the options (variable names) be sorted by frequency: 
#                 NULL (no sorting), 'descreasing', 'increasing'
# plot:           NULL (no barplot), 'frequency' (barplot of frequencies), 
#                 'percentage' (barplot of percentage of cases)
# names.arg       a vector of names used as option and to be plotted below each bar or group of bars.
#                 default is to use item names.
# ylab, ...       barplot options
#
multipleResponse <- function (data, items = NULL, regex = NULL, endorsedOption = 1, order=NULL, plot=NULL, names.arg=NULL, ylab=NULL, ...) 
{
  # From userfriendlyscience package
  if (is.null(regex) && is.null(items)) {
    items <- names(data)
  }
  else if (is.null(items)) {
    items <- grep(regex, names(data), value = TRUE)
  }
  if (!all(items %in% names(data))) {
    stop("You specified items that do not exist in the data you provided (specifically, ", 
         toString(items[!items %in% names(data)]), ").")
  }
  data = data[, items]
  nrOfEndorsements = sum(data == endorsedOption, na.rm = TRUE)
  endorsementsPerItem = colSums(data == endorsedOption, na.rm = TRUE)
  nrOfCases = sum(!apply(apply(data, 1, is.na), 2, all))
  totals = as.numeric(c(endorsementsPerItem, nrOfEndorsements))
  res <- data.frame(c(names(endorsementsPerItem), "Total"),
                    totals, (totals/nrOfEndorsements) * 100, 
                    (totals/nrOfCases) * 100)
  names(res) <- c("Option", "Frequency", "Perc. of responses", 
                  #                    paste0("Percentage of (", nrOfCases, ") cases"))
                  "Perc. of cases")
  # Option names
  n<-length(items)  
  if (!is.null(names.arg)) {
    res$Option<-c(names.arg,'Total')
  }
  # Sort
  if (!is.null(order)) {
    if (order == 'decreasing') {
      res<-res[c(order(res$Frequency[1:n], decreasing = TRUE),n+1),]
    }
    else if (order == 'increasing') {
      res<-res[c(order(res$Frequency[1:n], decreasing = FALSE),n+1),]
    }
  }
  # Plot
  if (!is.null(plot)) {
    if (plot == 'frequency') {
      if (is.null(ylab)) {ylab<-'Number of cases'}
      dataToPlot = res$Frequency[1:n]
      names(dataToPlot)<-res$Option[1:n]
      barplot(dataToPlot, ylab = ylab,...)
    } 
    else if (plot == 'percentage') {
      if (is.null(ylab)) {ylab<-'Percent of cases'}
      dataToPlot = res[,'Perc. of cases'][1:n]
      names(dataToPlot)<-res$Option[1:n]
      barplot(dataToPlot,ylab = ylab,...)
    }
  }
  cat(paste0("\nMultiple Response Variable (Number of cases: ", nrOfCases, ")\n\n"))
  return(res)
  
}